"""
Guessdomain python module
"""
import yaml
import os
from fnmatch import fnmatch
from netaddr import IPNetwork, IPAddress


def guess_domain(yaml_path='/etc/puppet/hieradata/'):
    """
    Guesses the domain of the machine and write it in hiera.
    """
    # Find the domain of the machine based on the IP.
    # https://network.cern.ch/sc/fcgi/sc.fcgi?Action=GetFile&file=ip_networks.html.
    yamlfile = yaml_path+"common.yaml"
    def_route = '/sbin/ip route list | grep default'
    dev_tmp = os.popen(def_route).read()
    dev = dev_tmp.split(" ")[4]
    if dev:
        ifconf4 = 'ip -4 addr show dev %s' % dev
        try:
            ipv4 = os.popen(ifconf4).read().split("\n")[1].strip(" ").split(" ")[1].split("/")[0]
        except IndexError:
            ipv4 = ''
        if ipv4 and fnmatch(ipv4, '*.*.*.*'):
            if fnmatch(ipv4, '10.14[4-9].*') or fnmatch(ipv4, '10.15[0-9].*'):
                dom = 'atlas'
            elif fnmatch(ipv4, '10.17[6-9].*') or fnmatch(ipv4, '10.18[0-9].*') or fnmatch(ipv4, '10.19[0-1].*'):
                dom = 'cms'
            elif fnmatch(ipv4, '137.138.20.24[0-1]') or fnmatch(ipv4, '137.138.20.25[0-5]'):
                dom = 'external'
            elif fnmatch(ipv4, '172.18.*'):
                dom = 'tn'
            else:
                dom = 'default'
        # At some point we will have only IPv6, guessing for both private and public prefixes.
        ifconf6 = 'ip -6 addr show dev %s' % dev
        try:
            ipv6 = os.popen(ifconf6).read().split("\n")[1].strip(" ").split(" ")[1].split("/")[0]
        except IndexError:
            ipv6 = ''
        if ipv6 and not dom:
            if IPAddress(ipv6) in IPNetwork('FD01:1458:0200::/44') or IPAddress(ipv6) in IPNetwork('2001:1458:0200::/44'):
                dom = 'gpn'
            elif IPAddress(ipv6) in IPNetwork('FD01:1458:0400::/44') or IPAddress(ipv6) in IPNetwork('2001:1458:0400::/44'):
                dom = 'tn'
            elif IPAddress(ipv6) in IPNetwork('FD01:1458:0600::/44') or IPAddress(ipv6) in IPNetwork('2001:1458:0600::/44'):
                dom = 'atlas'
            elif IPAddress(ipv6) in IPNetwork('FD01:1458:0700::/44') or IPAddress(ipv6) in IPNetwork('2001:1458:0700::/44'):
                dom = 'cms'
            elif IPAddress(ipv6) in IPNetwork('FD01:1458:0300::/44') or IPAddress(ipv6) in IPNetwork('2001:1458:0300::/44'):
                dom = 'lcg'
            elif IPAddress(ipv6) in IPNetwork('FD01:1458:0500::/44') or IPAddress(ipv6) in IPNetwork('2001:1458:0500::/44'):
                dom = 'alice'
            elif IPAddress(ipv6) in IPNetwork('FD01:1458:0800::/44') or IPAddress(ipv6) in IPNetwork('2001:1458:0800::/44'):
                dom = 'lhcb'
            else:
                dom = 'default'
        # Write the domain in common.yaml file
        with open(yamlfile, 'a') as common_file:
            mydata = {'cern_network_domain':dom}
            common_file.write(yaml.dump(mydata, default_flow_style=False))
        # Find the Hostname with nslookup
        nslook4 = '/usr/bin/nslookup %s 137.138.16.5 | /bin/grep "name ="' % ipv4
        try:
            fqdn = os.popen(nslook4).read().strip("\n").split("name = ")[1].strip(".")
            if fqdn:
                hostname = fqdn.split(".")[0]
            else:
                nslook6 = '/usr/bin/nslookup %s 137.138.16.5 | /bin/grep "name ="' % ipv6
                fqdn = os.popen(nslook6).read().strip("\n").split("name = ")[1].strip(".")
                hostname = fqdn.split(".")[0]
            return hostname
        except IndexError:
            print "[ERROR] Cannot guess the hostname of this machine.\n"
            raise SystemExit
    else:
        print "[ERROR] Cannot recognise the default network device.\n"
        raise SystemExit
